#
# Dynamic optimization of the Hicks CSTR problem
#
# Written by Antonio Flores T./CMU, 31 Jan, 2004
#
param NFE >= 1 integer       ; # Number of finite elements
param NCP >= 1, <= 5 integer ; # Number of collocation points
#
# Define initial values and final desired ones 
#
param Cinit >= 0  ;
param Tinit >= 0  ;
param Uinit >= 0  ;
param Cdes >= 0   ;
param Tdes >= 0   ;
param Udes >= 0   ;
param TransTime >= 0 ;
#
# Define specific parameters for Hicks multiplicity CSTR
#
param alpha >= 0  ;
param alpha1 >= 0 ;
param alpha2 >= 0 ;
param alpha3 >= 0 ;
param k10 >= 0    ;
param N >= 0      ;
param Cf > 0      ;
param J  > 0      ;
param Tf > 0      ;
param Tc > 0      ;
param yf >= 0     ;
param yc >= 0     ;
param theta > 0   ;
param r1   >= 0   ;
param r2   >= 0   ;
param r3   >= 0   ;
#
# Parameters for defining decision variables initial guesses
#
param POINT  ;
param SLOPEc ;
param SLOPEt ;
param SLOPEu ;
#
# Define dimensions for all indexed variables
#
set FE := 1..NFE ;
set CP := 1..NCP ;
param A {CP,CP}  ; # Collocation matrix
param H{FE};
#
# Define derivatives of the states evaluated at each collocation point
#
var Cdot {FE,CP} ;
var Tdot {FE,CP} ;
var TIME >= 0    ;
#
# Define the states value at the beginning of each finite element
#
var C0 {FE} >= 0.01, <= 1    ;
var T0 {FE} >= 0.01, <= 5    ;
#var U0 {FE} >= 0, <= 800 ;
var U0 {FE} >= 0, <= 2500 ;
#
# Define decision variables
#
var C {FE,CP} >= 0, <= 1   ; # Dimensionless concentration profile
var T {FE,CP} >= 0, <= 5   ; # Dimensionless temperature profile
#var U {FE,CP} >= 0, <= 1500 ; # Manipulated variable profile
var U {FE,CP} >= 0, <= 2500 ; # Manipulated variable profile
#var H {FE}    >= 0.1, <= 1 ; # Finite element length
#
# Objective function
#
minimize COST: 
sum{i in FE} (H[i]*sum{j in CP} ((
alpha1*(C[i,j]-Cdes)^2+alpha2*(T[i,j]-Tdes)^2+alpha3*(U[i,j]-Udes)^2)*A[j,NCP]));   
#
# Mass and Energy balance discretization
#
subject to FECOLc{i in FE,j in CP}:
C[i,j] = C0[i]+TIME*H[i]*sum{k in CP} A[k,j]*Cdot[i,k];

subject to FECOLt{i in FE,j in CP}:
T[i,j] = T0[i]+TIME*H[i]*sum{k in CP} A[k,j]*Tdot[i,k];
#
# Mass and Energy continuity constraints between finite elements
#
subject to CONc{i in FE diff{1}} :
C0[i] = C0[i-1]+TIME*H[i-1]*sum{k in CP} A[k,NCP]*Cdot[i-1,k] ;

subject to CONt{i in FE diff{1}} :
T0[i] = T0[i-1]+TIME*H[i-1]*sum{k in CP} A[k,NCP]*Tdot[i-1,k] ;
#
# Approximation of the Mass and Energy derivatives at each collocation point
#
subject to ODEc{i in FE, j in CP} :
Cdot[i,j] = (1-C[i,j])/theta-k10*exp(-N/T[i,j])*C[i,j] ; 

subject to ODEt{i in FE, j in CP} :
Tdot[i,j] = (yf-T[i,j])/theta+k10*exp(-N/T[i,j])*C[i,j]-alpha*U[i,j]*(T[i,j]-yc) ; 
#
# Initial conditions constraints
#
subject to IVc: C0[1] = Cinit ;
subject to IVt: T0[1] = Tinit ;
subject to IVu: U0[1] = Uinit ;
#
# End point conditions constraints
#
##subject to EPc: C[NFE,NCP] = Cdes ;
##subject to EPt: T[NFE,NCP] = Tdes ;
##subject to EPu: U[NFE,NCP] = Udes ;
#
# Constraint on the total transition time
#
#subject to TTT: sum {i in FE} H[i] <= theta ;

subject to TTT: TIME = TransTime ;

# -- End of the hicks.mod file --

