#
# Dynamic optimization of a tubular reactor with
# Diffusive and Convective Mass Transfer
#
# Written by Antonio Flores T., 5 March, 2008
#
param NFE >= 1 integer       ; # Number of finite elements
param NCP >= 1, <= 5 integer ; # Number of collocation points
param NPOC >=1, <= 10 integer; # Number of collocation points for discretizing spatial derivatives
#
# Define initial values and final desired ones 
#
param c2init    >= 0  ;
param c3init    >= 0  ; 
param c4init    >= 0  ;
param c5des     >= 0  ;
param peminit   >= 0  ;
param pemdes    >= 0  ;
param TransTime >= 0  ;
param r1        >= 0  ;
param r2        >= 0  ;
param r3        >= 0  ;
#
# Define specific parameters 
#
param alpha >= 0    ;
param alphac5 >= 0  ;
param alphapem >= 0 ;
param gamma >= 0    ;
param krate >= 0    ;
#
# Define dimensions for all indexed variables
#
set FE   := 1..NFE  ;
set CP   := 1..NCP  ;
set POC  := 1..NPOC ;
param A  {CP,CP}    ; # IRK matrix
param AL {POC,POC}  ; # Lagrange collocation matrix (first order spatial derivatives)
param BL {POC,POC}  ; # Lagrange collocation matrix (second order spatial derivatives)
param H  {FE}       ;
#
# Define derivatives of the states evaluated at each collocation point
#
var c2dot {FE,CP} ;
var c3dot {FE,CP} ;
var c4dot {FE,CP} ;
var TIME >= 0    ;
#
# Define the states value at the beginning of each finite element
#
var c02 {FE} >=0, <= 2;
var c03 {FE} >=0, <= 2;
var c04 {FE} >=0, <= 2;
#
# Define decision variables
#
var c1  {FE,CP} >=0, <= 2   ;
var c2  {FE,CP} >=0, <= 2   ;
var c3  {FE,CP} >=0, <= 2   ;
var c4  {FE,CP} >=0, <= 2   ;
var c5  {FE,CP} >=0, <= 2   ;
var pem {FE,CP} >=0, <= 150 ;
#
# Objective function
#
minimize COST: 
sum{i in FE} (H[i]*sum{j in CP} ((
alphac5*(c5[i,j]-c5des)^2+alphapem*(pem[i,j]-pemdes)^2)*A[j,NCP]));   
#
# Mass balance discretization
#
subject to FECOLc2{i in FE,j in CP}:
c2[i,j] = c02[i]+TIME*H[i]*sum{k in CP} A[k,j]*c2dot[i,k];

subject to FECOLc3{i in FE,j in CP}:
c3[i,j] = c03[i]+TIME*H[i]*sum{k in CP} A[k,j]*c3dot[i,k];

subject to FECOLc4{i in FE,j in CP}:
c4[i,j] = c04[i]+TIME*H[i]*sum{k in CP} A[k,j]*c4dot[i,k];
#
# Mass continuity constraints between finite elements
#
subject to CONc2{i in FE diff{1}} :
c02[i] = c02[i-1]+TIME*H[i-1]*sum{k in CP} A[k,NCP]*c2dot[i-1,k] ;

subject to CONc3{i in FE diff{1}} :
c03[i] = c03[i-1]+TIME*H[i-1]*sum{k in CP} A[k,NCP]*c3dot[i-1,k] ;

subject to CONc4{i in FE diff{1}} :
c04[i] = c04[i-1]+TIME*H[i-1]*sum{k in CP} A[k,NCP]*c4dot[i-1,k] ;
#
# Approximation of the Mass and Energy derivatives at each collocation point
#
subject to ODEc2{i in FE, j in CP} :
c2dot[i,j] = BL[2,1]*c1[i,j]+BL[2,2]*c2[i,j]+BL[2,3]*c3[i,j]+BL[2,4]*c4[i,j]+BL[2,5]*c5[i,j]
             -pem[i,j]*(AL[2,1]*c1[i,j]+AL[2,2]*c2[i,j]+AL[2,3]*c3[i,j]+AL[2,4]*c4[i,j]+AL[2,5]*c5[i,j])
             -pem[i,j]*alpha*krate*c2[i,j]^2 ;

subject to ODEc3{i in FE, j in CP} :
c3dot[i,j] = BL[3,1]*c1[i,j]+BL[3,2]*c2[i,j]+BL[3,3]*c3[i,j]+BL[3,4]*c4[i,j]+BL[3,5]*c5[i,j]
             -pem[i,j]*(AL[3,1]*c1[i,j]+AL[3,2]*c2[i,j]+AL[3,3]*c3[i,j]+AL[3,4]*c4[i,j]+AL[3,5]*c5[i,j])
             -pem[i,j]*alpha*krate*c3[i,j]^2 ;

subject to ODEc4{i in FE, j in CP} :
c4dot[i,j] = BL[4,1]*c1[i,j]+BL[4,2]*c2[i,j]+BL[4,3]*c3[i,j]+BL[4,4]*c4[i,j]+BL[4,5]*c5[i,j]
             -pem[i,j]*(AL[4,1]*c1[i,j]+AL[4,2]*c2[i,j]+AL[4,3]*c3[i,j]+AL[4,4]*c4[i,j]+AL[4,5]*c5[i,j])
             -pem[i,j]*alpha*krate*c4[i,j]^2 ;
#
# Additional algebraic equations resulting from discretizing the boundary conditions
#
subject to AEc1{i in FE, j in CP} :
AL[1,1]*c1[i,j]+AL[1,2]*c2[i,j]+AL[1,3]*c3[i,j]+AL[1,4]*c4[i,j]+AL[1,5]*c5[i,j] - pem[i,j]*(c1[i,j]-1) = 0 ;

subject to AEc5{i in FE, j in CP} :
AL[5,1]*c1[i,j]+AL[5,2]*c2[i,j]+AL[5,3]*c3[i,j]+AL[5,4]*c4[i,j]+AL[5,5]*c5[i,j] = 0 ;
#
# Initial conditions constraints
#
subject to IVc2: c02[1] = c2init ;
subject to IVc3: c03[1] = c3init ;
subject to IVc4: c04[1] = c4init ;
#
# Constraint on the total transition time
#
subject to TTT: TIME = TransTime ;

# -- End of the pde.mod file --

